//
//  EDAMBatchingNoteStoreClient.h
//  CoreNote
//
//  Created by Steve White on 6/5/12.
//  Copyright (c) 2012 Evernote Corporation. All rights reserved.
//

#import <EDAM/EDAM.h>

typedef NS_ENUM(NSInteger, EDAMBatchingType) {
    EDAMBatchingNotes,
    EDAMBatchingNotesAndResources,
    EDAMBatchingResourcesData,
    EDAMBatchingSearchContent,
    EDAMBatchingSearchText,
    EDAMBatchingRecoData,
    EDAMBatchingResourceAlternateData
};

typedef void (^EDAMFailureBlock)(NSException *exception);
typedef void (^EDAMContentEnumerationResultsBlock)(NSString *guid, id content, BOOL *stop);

@interface EDAMBatchingNoteStoreClient : EDAMNoteStoreClient

@property (nonatomic,copy) NSString* backgroundCacheDirectory;

- (void) enumerateContentForNotes:(NSArray *)noteGUIDs
          withAuthenticationToken:(NSString *)authenticationToken
                       usingBlock:(EDAMContentEnumerationResultsBlock)block
                     failureBlock:(EDAMFailureBlock)failureBlock;

- (void) enumerateContentForNotes:(NSArray *)noteGUIDs
                     andResources:(NSArray *)resourceGUIDs
          withAuthenticationToken:(NSString *)authenticationToken
                       usingBlock:(EDAMContentEnumerationResultsBlock)block
                     failureBlock:(EDAMFailureBlock)failureBlock;

- (void) enumerateDataForResources:(NSArray *)resourceGUIDs
           withAuthenticationToken:(NSString *)authenticationToken
                        usingBlock:(EDAMContentEnumerationResultsBlock)block
                      failureBlock:(EDAMFailureBlock)failureBlock;

- (void) enumerateSearchContentForNotes:(NSArray *)noteGUIDs
                           andResources:(NSArray *)resourceGUIDs
                withAuthenticationToken:(NSString *)authenticationToken
                             usingBlock:(EDAMContentEnumerationResultsBlock)block
                           failureBlock:(EDAMFailureBlock)failureBlock;

- (void) enumerateSearchTextForResources:(NSArray *)resourceGUIDs
                 withAuthenticationToken:(NSString *)authenticationToken
                              usingBlock:(EDAMContentEnumerationResultsBlock)block
                            failureBlock:(EDAMFailureBlock)failureBlock;

- (void) enumerateRecoDataForResources:(NSArray *)resourceGUIDs
               withAuthenticationToken:(NSString *)authenticationToken
                            usingBlock:(EDAMContentEnumerationResultsBlock)block
                          failureBlock:(EDAMFailureBlock)failureBlock;

- (void) enumerateResourceAlternateDataForResources:(NSArray *)resourceGUIDs
                            withAuthenticationToken:(NSString *)authenticationToken
                                         usingBlock:(EDAMContentEnumerationResultsBlock)block
                                       failureBlock:(EDAMFailureBlock)failureBlock;

- (NSArray *) getNotes:(NSArray *)noteGUIDs
      includingContent:(BOOL)withContent
         resourcesData:(BOOL)withResourcesData
  resourcesRecognition:(BOOL)withResourcesRecognition
resourcesAlternateData:(BOOL)withResourcesAlternateData
       applicationData:(BOOL)applicationData
usingAuthenticationToken:(NSString *)authenticationToken;


@end
