//
//  ENAssert.h
//  CoreNote
//
//  Created by Dirk Holtwick on 29.11.12.
//  Copyright (c) 2012 Evernote. All rights reserved.
//

#import "ENLogging.h"

#if ABORT_ON_ASSERT
  #define ENAbortInDebugMode abort();
#elif DEBUG
  #define ENAbortInDebugMode abort();
#else
  #define ENAbortInDebugMode
#endif

#define ENAssert(condition, fmt, ...) \
  do { \
    if (!(condition)) { \
      AppLogError(@"%@ <%@:%d> %s\n\n    %s failed assertion\n\n%@", \
        (fmt ? [NSString stringWithFormat:fmt, ##__VA_ARGS__]: @""), \
        [[NSString stringWithUTF8String:__FILE__] lastPathComponent], \
        __LINE__, \
        __PRETTY_FUNCTION__, \
        #condition, \
        [NSThread callStackSymbols] \
      ); \
      ENAbortInDebugMode \
    } \
  } while(0)

#define ENAssert0(condition) ENAssert((condition), @"Assertion")

#define ENAssertIsMainThread() ENAssert([NSThread isMainThread], @"Should always execute on main thread")

#define ENAssertIsNotMainThread() ENAssert(![NSThread isMainThread], @"Should never execute on main thread")

#define ENParameterAssert(condition) ENAssert((condition), @"Invalid parameter not satisfying: %s", #condition)
