//
//  ENCacheManager.h
//  CoreNote
//
//  Created by Steve White on 5/16/12.
//  Copyright (c) 2012 Evernote Corporation. All rights reserved.
//

#import <Foundation/Foundation.h>

@class ENAccountContext;
@class ENResourceMO;
@class ENNoteMO;

typedef enum {
  ENCacheManagerProgressGranularityNotebook = 0,
  ENCacheManagerProgressGranularityNote = 1,
} ENCacheManagerProgressGranularity;

extern NSString * const ENCacheManagerDidStartNotification;
extern NSString * const ENCacheManagerDidStopNotification;


@interface ENCacheManager : NSObject {
  __weak ENAccountContext *_accountContext;
  NSOperationQueue *_downloadQueue;
  NSString *_statusMessage;
  
  NSPredicate * _predicate;
  NSArray *_sortDescriptors;
  BOOL _downloadsNoteSearchText;
  BOOL _downloadsResourceSearchText;
  BOOL _downloadsResourceRecoData;
  BOOL _strictIntegrityChecks;
  
  NSArray *_downloadErrors;
  NSSet *_downloadingObjectIDs;
  ENCacheManagerProgressGranularity _progressGranularity;
  
  CFTimeInterval _lastResetTime;
  CFTimeInterval _lastFileSystemTime;
  
  int64_t _totalSize;
  int64_t _cachedSize;
  int64_t _uncachedSize;
  int64_t _downloadedSize;
  
  int64_t _incrementalSize;
  CFTimeInterval _lastIncrementalUpdate;
  
  BOOL _shouldStop;
#if TARGET_OS_IPHONE
  UIBackgroundTaskIdentifier _taskIdentifier;
#else
  BOOL _running;
#endif
}

@property (readonly, nonatomic) NSOperationQueue *downloadQueue;
@property (strong, nonatomic) NSString *statusMessage;

@property (strong, nonatomic) NSPredicate *predicate;
@property (strong, nonatomic) NSArray *sortDescriptors;
@property (assign, atomic) BOOL downloadsNoteSearchText;
@property (assign, atomic) BOOL downloadsResourceSearchText;
@property (assign, atomic) BOOL downloadsResourceRecoData;
@property (assign, nonatomic) BOOL strictIntegrityChecks;

@property (atomic, readonly) BOOL running;

@property (assign, atomic, readonly) int64_t totalSize;
@property (assign, atomic, readonly) int64_t cachedSize;
@property (assign, atomic, readonly) int64_t uncachedSize;
@property (assign, atomic, readonly) int64_t downloadedSize;

@property (strong, nonatomic, readonly) NSSet *downloadingObjectIDs;
@property (strong, nonatomic, readonly) NSArray *downloadErrors;
@property (assign, atomic) ENCacheManagerProgressGranularity progressGranularity;

- (id) initWithAccountContext:(ENAccountContext *)accountContext;
- (id) initWithAccountContext:(ENAccountContext *)accountContext
  listenForStartNotifications:(BOOL)listenForStartNotifications;

- (void) start;
- (void) stop;
- (void) cancel;
- (void) startInBackground;

// cache directory to store request info for background transfers

- (NSString *) backgroundCacheDirectory;

@end
