//
//  ENCacheProcessor.h
//  CoreNote
//
//  Created by Steve White on 2/13/14.
//  Copyright (c) 2013 Evernote Corporation. All rights reserved.
//

#import <Foundation/Foundation.h>


extern NSString * const kServiceEntityIDKey;
extern NSString * const kNoteIDKey;
extern NSString * const kGUIDKey;


@class ENAccountContext;
@class ENNoteMO;
@class ENResourceMO;
@class NSManagedObjectID;

@interface ENCacheProcessor : NSObject

- (id)initWithAccountContext:(ENAccountContext*)context;

//
// These methods are typically invoked from the TSessionHTTPClient
// The NSDictionary key is expected to be the entity's guid,
// and the value is expected to be the content.
//
- (void) processNoteContentResults:(NSDictionary *)noteContents;
- (void) processResourceContentResults:(NSDictionary *)resourceContents;
- (void) processSearchContentByGuidForNotes:(NSDictionary *)notes
                                  resources:(NSDictionary *)resources;

//
// These are typically invoked externally from various cache operations
// And internally from the above
//
- (BOOL) processNoteContent:(NSString *)content
                    forNote:(NSDictionary *)note  // from +dictionaryFromNote:
                      error:(NSError *__autoreleasing *)error;

- (BOOL) processResourceContent:(id)content
                    forResource:(NSDictionary *)resource  // from +dictionaryFromResource
                          error:(NSError *__autoreleasing *)error;

//
// Notes and resources are arrays of dictionaries.
// The dictionary is expected to hold two values,
// ENBatchSearchTextDownloadContentKey    : <search content>
// ENBatchSearchTextDownloadEntityDataKey : <data from +entityDataForServiceEntity: >
//
- (BOOL) processSearchContentForNotes:(NSArray *)notes
                            resources:(NSArray *)resources
                                error:(NSError *__autoreleasing *)error;


//
// This is invoked from either TSessionHTTPClient or the cache operation
//
- (void) processResourceRecoResults:(NSDictionary *)resourceRecos;


//
// Results: dictionary
// key: resource guid
// value: resource alternate data
- (void) processResourceAlternateDataResults:(NSDictionary *)results;


//
// Class methods are a-w-e-s-o-m-e!
//
+ (NSDictionary *) entityDataForServiceEntity:(id)serviceEntity;
+ (NSDictionary*)dictionaryFromNote:(ENNoteMO*)note;
+ (NSDictionary*)dictionaryFromResource:(ENResourceMO*)resource;
+ (NSManagedObjectID *) objectIdFromError:(id)anError;
+ (BOOL) isAuthenticationError:(id)anError;
+ (BOOL) isUnrecoverableError:(id) anError;

@end
