//
//  ENLogging.h
//  CoreNote
//
//  Created by Andrew McGeachie on 10/1/13.
//  Copyright (c) 2013 Evernote. All rights reserved.
//

#ifndef ENLogging_HEADER_GUARD
#define ENLogging_HEADER_GUARD

#if defined USE_LUMBERJACKLOGGER && USE_LUMBERJACKLOGGER == 1
  #include <Lumberjack/DDLog.h>
  #include <Lumberjack/DDASLLogger.h>
  #include <Lumberjack/DDTTYLogger.h>
  #include <Lumberjack/DDFileLogger.h>
  #if DEBUG
    static const int ddLogLevel = LOG_LEVEL_VERBOSE;
    #undef LOG_ASYNC_ENABLED
    #define LOG_ASYNC_ENABLED NO
  #else
    static const int ddLogLevel = LOG_LEVEL_INFO;
  #endif

  #define AppLogDebug(...)  DDLogCDebug(__VA_ARGS__)
  #define AppLogInfo(...)   DDLogCInfo(__VA_ARGS__)
  #define AppLogWarn(...)   DDLogCWarn(__VA_ARGS__)
  #define AppLogError(...)  DDLogCError(__VA_ARGS__)

#elif defined USE_NSLOG && USE_NSLOG == 1
  #define AppLogDebug(...)  NSLog(__VA_ARGS__)
  #define AppLogInfo(...)   NSLog(__VA_ARGS__)
  #define AppLogWarn(...)   NSLog(__VA_ARGS__)
  #define AppLogError(...)  NSLog(__VA_ARGS__)

#else // default logger if nothing else is defined
  #define USE_GTMLOGGER 1
  #import "GTMLogger.h"
  #define AppLogDebug(...)  \
      [[GTMLogger sharedLogger] logFuncDebug:__func__ msg:__VA_ARGS__]
  #define AppLogInfo(...)   \
      [[GTMLogger sharedLogger] logFuncInfo:__func__ msg:__VA_ARGS__]
  #define AppLogWarn(...)  \
      [[GTMLogger sharedLogger] logFuncError:__func__ msg:__VA_ARGS__]
  #define AppLogError(...)  \
      [[GTMLogger sharedLogger] logFuncError:__func__ msg:__VA_ARGS__]

#endif

@interface ENLogging : NSObject

+ (NSFileHandle *) fileHandleForLogAtPathRotatingIfNecessary: (NSString *) filePath;

#if defined USE_GTMLOGGER && USE_GTMLOGGER == 1
+ (void) setupGTMLoggingWithPath: (NSString *) logFile;
+ (void) setupGTMLoggingWithPath: (NSString *) logFile verboseLogging: (BOOL) verboseLogging;
#endif

@end


#endif // ENLogging_HEADER_GUARD