#ifndef ENNoteMO_HEADER_GUARD
#define ENNoteMO_HEADER_GUARD

#import "_ENNoteMO.h"
#import <CoreLocation/CoreLocation.h>

@class ENNotebookMO;
@class ENResourceMO;
@class ENTagMO;
@class EDAMNote;

extern NSString * const ENNoteLocationUnknown;

extern NSString * const ENNoteSourceApplicationBusinessCards;
extern NSString * const ENNoteSourceApplicationMoleskine;
extern NSString * const ENNoteSourceApplicationPostIt;


@interface ENNoteMO : _ENNoteMO {}

@property (copy, nonatomic) CLLocation *location;
@property (nonatomic) int32_t creatorId;
@property (nonatomic) int32_t lastEditorId;

+ (void) registerENMLTransformerClass:(Class)transformerClass;
+ (Class) enmlTransformerClass;

+ (NSPredicate *) predicateForNotesInTrash;
+ (NSPredicate *) predicateForNotesInStack:(NSString *)stack;
+ (NSPredicate *) predicateForNotesInNotebook:(ENNotebookMO *)notebook;
+ (NSPredicate *) predicateForNotesWithTag:(ENTagMO *)tag;
+ (NSPredicate *) predicateForNotesWithGeolocation;
+ (NSPredicate *) predicateForNotesInRegion:(CLRegion *)region;

- (ENResourceMO *) resourceWithGUID:(NSString *)guid;
- (ENResourceMO *) resourceWithHash: (NSData *) hash;

- (NSString *) backingFileExtension;
- (NSString *) absolutePathForBackingFile;
- (NSArray *) allBackingFilePaths;
@property (readonly,nonatomic) NSString * absolutePathForBackingDirectory;

// Returns the total size of note, including resources
- (uint32_t) totalNoteSize;

// Returns the cached size of the note, including resources
- (uint32_t) cachedNoteSize;

// Returns the cache status of the note and it's resources
// ENManagedObjectCachingStatusComplete means that everything
// exists.  
- (ENManagedObjectCachingStatus) totalCacheStatus;

- (void) invalidateLocalContent;
- (BOOL) needsSearchContent;

/**
 * Convenience property
 */
@property (readonly,nonatomic) BOOL isBusinessNote;

#pragma mark - Note sync callbacks

/**
 *  This callback will be invoked before this note instance is updated with new server values.
 *
 *  @param serverNote the server note data that was just synced from the server
 */
- (void) willUpdateFromServerNote:(EDAMNote *)serverNote;

/**
 *  This callback will be invoked after this note instance was updated with new server values.
 *
 *  @param serverNote the server note data that was just synced from the server
 */
- (void) didFinishUpdatingFromServiceNote:(EDAMNote *)serverNote;

/**
 *  This callback will be invoked before the serverNote instance is updated with new values from this instance.
 *
 *  @param serverNote the server note data that is about to be synced with server
 */
- (void) willBeginPreparingNoteForUpload:(EDAMNote *)serverNote;

/**
 *  This callback will be invoked after the serverNote instance was updated with new values from this instance.
 *
 *  @param serverNote the server note data that is about to be synced with server
 */
- (void) didFinishPreparingNoteForUpload:(EDAMNote *)serverNote;

#pragma mark - Server download progress

// you need to enable track download progress before the API below will have any affect
+ (void) setTrackDownloadProgress:(BOOL)enableProgress;
+ (BOOL) trackDownloadProgress;

// compute the server payload left to download, which can be less than total note size
- (uint32_t) totalDownloadSize;

// prime and reset for the next download session; must be on main thread!
- (void) resetDownloadProgress:(uint32_t)totalDownloadSize;

// true iff we're currently downloading note content from the server; must be on main thread!
@property (nonatomic) BOOL isLoading;
// percent loaded in this session; must be on main thread!
@property (nonatomic) uint32_t percentLoaded;
// bytes downloaded already in this session; must be on main thread!
@property (nonatomic) uint32_t bytesDownloaded;
// total bytes we expect to download this session; must be on main thread!
@property (nonatomic) uint32_t downloadSize;

#pragma mark

@property (copy, nonatomic) NSDictionary *lastKnownServerValues;


/**
 * Handy for logging
 */
@property (readonly,nonatomic) NSString * shortDescription;


@end


/**
 *  optional methods on ENNoteMO that will be used if available
 */
@interface ENNoteMO(OptionalMethods)
- (NSString *) absolutePathForRawServerFile;
- (NSString *) enmlContent;
- (void) setEnmlContent:(NSString *)enmlContent;
- (void) setSearchText:(NSString *) searchText;
@end


#endif
