#ifndef ENNotebookMO_HEADER_GUARD
#define ENNotebookMO_HEADER_GUARD

#import "_ENNotebookMO.h"

@class ENSharedNotebookMO;
@class EDAMNotebook;


typedef enum {
  ENNotebookRestrictionsCanReadNotes = 1 << 0,
  ENNotebookRestrictionsCanCreateNotes = 1 << 1,
  ENNotebookRestrictionsCanUpdateNotes = 1 << 2,
  ENNotebookRestrictionsCanExpungeNotes = 1 << 3,
  ENNotebookRestrictionsCanShareNotes = 1 << 4,
  ENNotebookRestrictionsCanEmailNotes = 1 << 5,
  ENNotebookRestrictionsCanSendMessageToRecipients = 1 << 6,
  ENNotebookRestrictionsCanUpdateNotebook = 1 << 7,
  ENNotebookRestrictionsCanExpungeNotebook = 1 << 8,
  ENNotebookRestrictionsCanSetDefaultNotebook = 1 << 9,
  ENNotebookRestrictionsCanSetNotebookStack = 1 << 10,
  ENNotebookRestrictionsCanPublishToPublic = 1 << 11,
  ENNotebookRestrictionsCanPublishToBusinessLibrary = 1 << 12,
  ENNotebookRestrictionsCanCreateTags = 1 << 13,
  ENNotebookRestrictionsCanUpdateTags = 1 << 14,
  ENNotebookRestrictionsCanExpungeTags = 1 << 15,
  ENNotebookRestrictionsCanSetParentTag = 1 << 16,
  ENNotebookRestrictionsCanCreateSharedNotebooks = 1 << 17,
} ENNotebookRestrictions;

@interface ENNotebookMO : _ENNotebookMO {}

+ (NSPredicate *) predicateForRestriction:(ENNotebookRestrictions)restriction value:(BOOL)value;

@property (strong, nonatomic) NSString *displayName;
@property (strong, nonatomic) NSString *displayStack;

@property (assign, nonatomic) BOOL canReadNotes;
@property (assign, nonatomic) BOOL canCreateNotes;
@property (assign, nonatomic) BOOL canUpdateNotes;
@property (assign, nonatomic) BOOL canExpungeNotes;
@property (assign, nonatomic) BOOL canShareNotes;
@property (assign, nonatomic) BOOL canEmailNotes;
@property (assign, nonatomic) BOOL canSendMessageToRecipients;
@property (assign, nonatomic) BOOL canUpdateNotebook;
@property (assign, nonatomic) BOOL canExpungeNotebook;
@property (assign, nonatomic) BOOL canSetDefaultNotebook;
@property (assign, nonatomic) BOOL canSetNotebookStack;
@property (assign, nonatomic) BOOL canPublishToPublic;
@property (assign, nonatomic) BOOL canPublishToBusinessLibrary;
@property (assign, nonatomic) BOOL canCreateTags;
@property (assign, nonatomic) BOOL canUpdateTags;
@property (assign, nonatomic) BOOL canExpungeTags;
@property (assign, nonatomic) BOOL canSetParentTag;
@property (assign, nonatomic) BOOL canCreateSharedNotebooks;

@property (nonatomic, readonly) BOOL isBusinessUserNotebook;
@property (nonatomic, readonly, getter = isJoinedNotebook) BOOL joinedNotebook;
@property (nonatomic, readonly) NSInteger numberOfRecipients;

- (ENSharedNotebookMO *) sharedNotebookWithShareKey:(NSString *)shareKey;
- (void) noteChangedActiveValue:(ENNoteMO *)note;

/**
 *  This callback will be invoked before this notebook instance is updated with new server values.
 *
 *  @param serverNotebook the server notebook data that was just synced from the server
 */
- (void) willUpdateFromServerNotebook:(EDAMNotebook *)serverNotebook;


/**
 *  This callback will be invoked after this notebook instance was updated with new server values.
 *
 *  @param serverNotebook the server notebook data that was just synced from the server
 */
- (void) didUpdateFromServerNotebook:(EDAMNotebook *)serverNotebook;


@end

#endif
