#ifndef ENResourceMO_HEADER_GUARD
#define ENResourceMO_HEADER_GUARD

#import "_ENResourceMO.h"
#import "ENContentTransformer.h"

@class CLLocation;

extern int const ENResourceMOInkSliceHeight;
extern int const ENResourceMOInkSliceWidth;

@interface ENResourceMO : _ENResourceMO {}

@property (copy, nonatomic) CLLocation *location;

- (NSURL *) evernoteAppURLForInkSlice:(NSUInteger)inkSlice;
- (NSURL *) evernoteThumbnailURL;

- (NSString *) bestGuessMimeType;
- (NSString *) absolutePathForBackingFile;
- (NSString *) absolutePathForBackingFileSlice:(NSUInteger)slice;
- (NSArray *) allBackingFilePaths;
- (NSString *) backingFileExtension;

- (BOOL) isInkResource;
- (NSUInteger) numberOfInkSlices;

- (BOOL) isIndexable;
- (BOOL) needsSearchText;
- (BOOL) needsRecoText;

// Both of these methods are disk io-bound...
- (void) writeDataToBackingFile:(NSData *)data;
- (NSData *)dataFromBackingFile; // does not cache in memory. pulls directly from disk on every call for the same resource.

- (void)invalidateSearchText;

// A transformer that is capable of converting server
// data to local data (e.g. splitting up a PNG file
// into several files), as well as converting local
// data back into server data (e.g. concatenating
// several files back into a single PNG file)
- (id<ENContentTransformer>) contentTransformer;

/** 
 * The actual class (possibly a subclass of ENResourceMO) that implements resource at runtime.
 */
+ (Class) runtimeImplementationClass;

/**
 * Whether the runtime resource implementation wants resourceAlternateData.
 * (Base class ENResourceMO returns NO)
 */
+ (BOOL) wantsResourceAlternateData;


/**
 * Whether the runtime resource implementation wants resourceRecoText
 */
+ (BOOL) wantsResourceRecoText;


/**
 * Set alternate resource metadata (hash + length).
 * (Base class ENResourceMO does nothing)
 */
- (void) setResourceAlternateDataHash:(NSData *)hash length:(NSUInteger)length;
 

/** 
 * get/set alternate resource data.
 * (Base class ENResourceMO does nothing)
 */
@property (copy,nonatomic) NSData * resourceAlternateData;


/**
 * Whether this resource instance needs resource alternate data.
 * (Base class ENResourceMO returns NO)
 */
- (BOOL) needsResourceAlternateData;


/**
 * Whether this resource instance needs ink resource content (in addition to ink-rendering slices).
 * (Base class ENResourceMO returns NO)
 */
- (BOOL) needsInkResourceContent;


/**
 * Whether this resource's ink-rendering slices all are present.
 * If this resource is not an ink resource, returns YES.
 */
- (BOOL) allInkSlicesPresent;


/**
 * Absolute path to an optional alternate complete ink rendering to use in place of slices.
 * (Base class ENResourceMO returns nil)
 */
- (NSString *) absolutePathForStitchedInkRendering;


/**
 * Update our cache status to complete or incomplete based on what data we have cached
 * locally
 */
- (void) updateCacheStatus;

@end


@interface ENResourceMO (OptionalMethods)
- (void) setRecoText: (NSString *) recoXML;
- (NSString *) recoXmlContents;
- (void) setRecoXmlContents:(NSString *)recoXmlContents;
- (void) setSearchText:(NSString *) searchText;
@end



#endif