//
//  ENSyncState.h
//  Evernote
//
//  Created by Steve White on 12/11/10.
//  Copyright 2010 Evernote Corporation. All rights reserved.
//


#ifndef ENSyncState_HEADER_GUARD
#define ENSyncState_HEADER_GUARD

#import <Foundation/Foundation.h>

//
// This information used to be a managed object stored in CoreData
// However with the frequency in which it was updated the associated
// saves with it took unbearable amounts of time (2-3 secs). So
// it was moved out and persisted to disk, as to not interfere 
// with any CoreData performance.
//

extern NSString * const ENSyncStateValueChangedNotification;

extern NSString * const ENSyncStateNeedsIntegrityCheckKey;

extern NSString * const ENSyncStateEDAMUserKey;
extern NSString * const ENSyncStateUserIDKey;
extern NSString * const ENSyncStateUserEmailKey;
extern NSString * const ENSyncStateUsernameKey;
extern NSString * const ENSyncStateUserFullNameKey;
extern NSString * const ENSyncStateIncomingEmailAddressKey;
extern NSString * const ENSyncStateRecentMailedAddressesKey;
extern NSString * const ENSyncStateUserLastUpdatedKey;

extern NSString * const ENSyncStateCanShowInAppPurchasingKey;
extern NSString * const ENSyncStatePremiumServiceStart;
extern NSString * const ENSyncStatePremiumExpirationDate;
extern NSString * const ENSyncStatePremiumRecurring;
extern NSString * const ENSyncStatePremiumCommerceService;
extern NSString * const ENSyncStatePremiumBeginsExpiration;
extern NSString * const ENSyncStatePremiumCompletesExpiration;
extern NSString * const ENSyncStateHasDisplayedPremiumWillExpire;
extern NSString * const ENSyncStateHasDisplayedPremiumDidExpire;

extern NSString * const ENSyncStateLastSyncTimeKey;
extern NSString * const ENSyncStateUploadLimitEndKey;
extern NSString * const ENSyncStateMemberSinceKey;
extern NSString * const ENSyncStateServerUploadUsedKey;
extern NSString * const ENSyncStateUploadLimitKey;
extern NSString * const ENSyncStateAccountSizeKey;
extern NSString * const ENSyncStateLastSyncUpdateCountKey;
extern NSString * const ENSyncStateShardIdKey;
extern NSString * const ENSyncStateAccountTypeKey;
extern NSString * const ENSyncStateLocalUploadUsedKey;
extern NSString * const ENSyncStateIncrementalSyncAtUSNKey;
extern NSString * const ENSyncStateFullSyncAtUSNKey;

extern NSString * const ENSyncStateReminderEmailConfigKey;
extern NSString * const ENSyncStateTimeZoneNameKey;

extern NSString * const ENSyncStateBusinessIdKey;
extern NSString * const ENSyncStateBusinessNameKey;
extern NSString * const ENSyncStateBusinessActivatedKey;
extern NSString * const ENSyncStateBusinessActivatedOnceKey;

extern NSString * const ENSyncStateMetadataKey;
extern NSString * const ENSyncStateObjectFailuresKey;
extern NSString * const ENSyncStateOperationsKey;
extern NSString * const ENSyncStateChildrenKey;

extern NSString * const ENSyncStateNotebooksVersionKey;
extern NSString * const ENSyncStateLinkedNotebooksVersionKey;
extern NSString * const ENSyncStateSavedSearchesVersionKey;
extern NSString * const ENSyncStateSharedNotebooksVersionKey;
extern NSString * const ENSyncStateTagsVersionKey;
extern NSString * const ENSyncStateUserProfilesVersionKey;

@class EDAMUser;
@class ENSyncOperation;

@interface ENSyncState : NSObject<NSCoding>

@property (strong, nonatomic) NSString *file;

@property (assign, nonatomic) BOOL needsIntegrityCheck;

@property (strong, nonatomic) EDAMUser *edamUser;
@property (assign, nonatomic) int32_t userID;
@property (strong, nonatomic) NSString *userEmail;
@property (strong, nonatomic) NSString *username;
@property (strong, nonatomic) NSString *userFullName;
@property (strong, nonatomic) NSString *incomingEmailAddress;
@property (strong, nonatomic) NSArray *recentMailedAddresses;
@property (strong, nonatomic) NSDate *userLastUpdated;

@property (assign, nonatomic) BOOL canShowInAppPurchasing;
@property (strong, nonatomic) NSDate *premiumServiceStart;
@property (strong, nonatomic) NSDate *premiumExpirationDate;
@property (assign, nonatomic, getter=isPremiumRecurring) BOOL premiumRecurring;
@property (strong, nonatomic) NSString *premiumCommerceService;
@property (strong, nonatomic) NSDate *premiumBeginsExpiration;
@property (strong, nonatomic) NSDate *premiumCompletesExpiration;
@property (assign, nonatomic) BOOL hasDisplayedPremiumWillExpire;
@property (assign, nonatomic) BOOL hasDisplayedPremiumDidExpire;

@property (strong, nonatomic) NSDate *lastSyncTime;
@property (strong, nonatomic) NSDate *uploadLimitEnd;
@property (strong, nonatomic) NSDate *memberSince;
@property (nonatomic, assign) long long serverUploadUsed;
@property (nonatomic, assign) long long uploadLimit;
@property (nonatomic, assign) long long quotaRemainingOverride;  // INT64_MAX: not set
@property (nonatomic, assign) int accountSize;
@property (nonatomic, assign) unsigned int lastSyncUpdateCount;
@property (strong, nonatomic) NSString *shardId;
@property (nonatomic, assign) short accountType;
@property (nonatomic, assign, readonly, getter=isPremiumAccount) BOOL premiumAccount;
@property (nonatomic, assign) long long localUploadUsed;
@property (nonatomic, assign) unsigned int incrementalSyncAtUSN;
@property (nonatomic, assign) unsigned int fullSyncAtUSN;

@property (strong, nonatomic) NSNumber *reminderEmailConfig;
@property (strong, nonatomic) NSString *timeZoneName;

@property (strong, nonatomic) NSNumber *businessId;
@property (strong, nonatomic) NSString *businessName;
@property (strong, nonatomic) NSNumber *businessActivated;
@property (strong, nonatomic) NSNumber *businessActivatedOnce;

@property (strong, nonatomic) NSDictionary *metadata;
@property (strong, nonatomic) NSDictionary *objectFailures;

@property (nonatomic, assign) int notebooksVersion;
@property (nonatomic, assign) int linkedNotebooksVersion;
@property (nonatomic, assign) int savedSearchesVersion;
@property (nonatomic, assign) int sharedNotebooksVersion;
@property (nonatomic, assign) int tagsVersion;
@property (nonatomic, assign) int userProfilesVersion;

@property (strong, nonatomic) NSDate *lastUserInfoUpdate;
@property (assign, nonatomic) BOOL hasCheckedEntityVersions;

@property (strong, nonatomic) NSString *hostUUID;

- (id) initWithContentsOfFile:(NSString *)file
                 restoredData:(BOOL *)restoredData;

- (id) metadataObjectForKey:(NSString *)key;
- (void) setMetadataObject:(id)object forKey:(NSString *)key;

- (NSDate *) lastFailureDateForObjectWithID:(id)objectID;
- (void) setFailureDate:(NSDate *)failureDate forObjectWithID:(id)objectID;

- (NSArray *) operations;
- (void) addOperation:(ENSyncOperation *)syncOperation;
- (void) removeOperation:(ENSyncOperation *)syncOperation;

- (ENSyncState *) businessSyncState;
- (ENSyncState *) syncStateForNotebookWithGUID:(NSString *)notebookGUID;
- (void) removeSyncStateForNotebookWithGUID:(NSString *)notebookGUID;

- (void) updateWithEDAMUserObject:(EDAMUser *)edamUser;

#pragma mark - Testing Simulation
@property (assign, nonatomic) BOOL simulateFailureOnNextNoteUpdate;

// set a key/value on the sync state and all child/biz sync states
- (void) setValueRecursively:(id)value forKey:(NSString *)key;

@end



#endif