//
//  NSURLAdditions.h
//  EverNote
//
//  Created by Andrew McGeachie on 6/3/08.
//  Copyright 2008 Evernote Corporation. All rights reserved.
//


@interface NSURL (NSURLAdditions)

/**
 * Return a new URL identical to this one except its using https.
 * If this URL is already using https then it is returned.
 * If this URL does not use http then nil is returned.
 */
- (NSURL *) URLByMakingThisURLUseHTTPS;


/**
 * Return a new URL identical to this one except its using http.
 * If this URL is already using http then it is returned.
 * If this URL does not use https then nil is returned.
 */
- (NSURL *) URLByMakingThisURLUseHTTP;

/**
 * Return value of a query parameter identified by queryKey.
 * If none were find, return NIL.
 */
- (NSString *) queryValueForKey:(NSString *)queryKey;

- (NSDictionary *)dictionaryFromQueryComponents;

/**
 * Returns query string up to hash mark (if one exists)
 */
- (NSString *) queryUpToHash;

- (NSURL *) urlByAddingQueryParameters:(NSDictionary *)queryParameters;
- (NSURL *) URLByDeletingQueryParameters;
- (NSURL *) URLWithQueryParameters:(NSDictionary *)queryParameters;
- (NSURL *) URLByAddingQueryParameters:(NSDictionary *)queryParameters
               toDeepestParameterNamed:(NSString *)parameterName;

@end
