//
//  ENCustomImageCellInternal.h
//  EverNote
//
//  Created by Matt Sarnoff on 7/9/12.
//  Copyright (c) 2012 Evernote. All rights reserved.
//
//  Code shared by ENCustomThreePartImageButtonCell and ENCustomImagePopUpButtonCell.

typedef enum
{
  kENCustomImageCellNormalState,
  kENCustomImageCellOnState,
  kENCustomImageCellPressedState,
  kENCustomImageCellOnPressedState,
  kENCustomImageCellNumStates,
} ENCustomImageCellState;


typedef enum
{
  kENCustomImageCellLeftPart = 0,
  kENCustomImageCellMiddlePart,
  kENCustomImageCellRightPart,
  kENCustomImageCellNumParts
} ENCustomImageCellPart;



#pragma mark Used by ENCustomThreePartImageButtonCell and ENCustomImagePopUpButtonCell. Do not instantiate!
@interface ENCustomImageCellInternal : NSObject <NSCopying>
{
  // left/middle/right, off/on/pressed
  NSImage *mBezelImages[kENCustomImageCellNumStates][kENCustomImageCellNumParts];
  BOOL mNoLeftPart;
  BOOL mNoRightPart;
}

@property (assign) BOOL noLeftPart;
@property (assign) BOOL noRightPart;


/**
 * Sets the bezel image for a specific state and part.
 */
- (void)setImage:(NSImage *)image forState:(ENCustomImageCellState)state part:(ENCustomImageCellPart)part;


/**
 * Retrieves the bezel image for a specific state and part.
 */
- (NSImage *)imageForState:(ENCustomImageCellState)state part:(ENCustomImageCellPart)part;


/**
 * Sets the three-part bezel image for the normal state.
 */
- (void)setLeftImage:(NSString *)leftImageName middleImage:(NSString *)middleImageName rightImage:(NSString *)rightImageName;


/**
 * Sets the three-part bezel image for the pressed state.
 */
- (void)setPressedLeftImage:(NSString *)leftImageName middleImage:(NSString *)middleImageName rightImage:(NSString *)rightImageName;


/**
 * Set the three-part bezel images for all states.
 * If namePrefix is "image", this looks for images named:
 *   image-normal-left, image-normal-middle, image-normal-right
 *   image-on-left, image-on-middle, image-on-right (optional)
 *   image-pressed-left, image-pressed-middle, image-pressed-right
 *   image-on-pressed-left, image-on-pressed-middle, image-on-pressed-right (optional)
 *
 * If a -left or -right image isn't found, it will fall back to the -middle image for that state.
 * This allows creation of segmented buttons without duplicating artwork.
 */
- (void)setImagesWithNamePrefix:(NSString *)namePrefix;


- (void)drawState:(ENCustomImageCellState)state withFrame:(NSRect)frame inView:(NSView *)view enabled:(BOOL)cellIsEnabled;


- (void)drawLeftPartState:(ENCustomImageCellState)leftState middlePartState:(ENCustomImageCellState)middleState rightPartState:(ENCustomImageCellState)rightState withFrame:(NSRect)frame inView:(NSView *)view enabled:(BOOL)cellIsEnabled;


- (CGFloat)leftImageWidthForState:(ENCustomImageCellState)state;


- (CGFloat)rightImageWidthForState:(ENCustomImageCellState)state;


- (CGFloat)imageHeight;

@end