//
//  ENResourceMO+ENSharedUtilities.h
//  ENSharedUtilities
//
//  Created by Andrew McGeachie on 12/20/13.
//  Copyright (c) 2013 Evernote. All rights reserved.
//

#import <AppKit/AppKit.h>
#import <CoreNote/CoreNote.h>

@interface ENResourceMO (ENSharedUtilities)

/**
 * Whether the Mac client offers external-edit support for the given file extension.
 */
+ (BOOL)isSupportedPackageDocumentExtension:(NSString *)extension;

/**
 * See if we have a pdf for displaying our content inline
 */
- (BOOL) hasInlinePDF;

  
/** Whether this resource is a PDF. */
- (BOOL) isPDF;


/**
 * Whether this resource is a playable audio file.
 */
- (BOOL) isPlayableAudioFile;


/**
 * YES if we have this resource data locally
 */
- (BOOL) hasData;


/**
 * This method tweaks the dragging pasteboard to enable drag and drop of image resources
 * to Safari, Chrome, Firefox, and Mailplane3.
 * It will add the filename (sanitized, of course) to the pasteboard instead of the UUID-style actual filename.
 * https://evernote.jira.com/browse/MACOSNOTE-6667
 */
- (void)addFilenamesForResourceToPasteboard:(NSPasteboard *)pboard;


/**
 * Whether this resource appears to be a zipped package document (e.g. an Apple Pages document,
 * where the "document" consists of multiple files in a directory, zipped up).
 */
- (BOOL)isZippedPackageDocument;


/**
 * This resource file's document extension, if it is a zipped package document.
 * For example, this will return "pages" for a resource with filename "foo.pages.zip".
 * NOTE: a non-nil return value from this method DOES NOT by itself indicate that this resource
 * should be treated as a zipped package document -- check isZippedPackageDocument for that.
 */
- (NSString *)zippedPackageDocumentExtension;

 
/** Whether this resource is an image. */
- (BOOL)isImage;


/**
 * File for storing our inline pdf (if applicable, void where prohibited)
 */
- (NSString *) inlinePDFFile;


@end
