//
//  ENSystemUtils.h
//  EverNote
//
//  Created by Andrew McGeachie on 7/1/11.
//  Copyright 2011-2012 Evernote. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CoreNote/ENSystemUtils.h>


@interface ENSystemUtils (ENSharedUtilities)


/**
 * Get a set of ProcessSerialNumber objects representing processes whose names match the parameter.
 * @param names Set of names to expect from e.g. CopyProcessName.
 * @return A (possibly empty) set of NSData objects, each of which contains a ProcessSerialNumber.
 */
+ (NSSet *)psnsForProcessesWithNames:(NSSet *)names;


/**
 * Get the bundle path, if any, for the application running with the given PSN.
 */
+ (NSString *)bundlePathForPSN:(ProcessSerialNumber *)psn;


/**
 * Are we using the Mac App-Store for purchases?
 */
+ (BOOL) usingMacAppStore;


/**
 * Returns if the app is running in a sandbox
 */
+ (BOOL)runningInSandbox;


@end
