//
//  GAMFields.h
//  GAMKit
//
//  Created by Isaac Wankerl on 11/12/13.
//  Copyright (c) 2013 Evernote Corp. All rights reserved.
//

#import <Foundation/Foundation.h>

/*!
 These fields can be used for the wire format parameter names required by
 the |GAMTracker| get, set and send methods as well as the set methods in the
 |GAMDictionaryBuilder| class.
 */

extern NSString *const kGAMHitType;
extern NSString *const kGAMTrackingId;
extern NSString *const kGAMClientId;
extern NSString *const kGAMAnonymizeIp;
extern NSString *const kGAMSessionControl;
extern NSString *const kGAMScreenResolution;
extern NSString *const kGAMViewportSize;
extern NSString *const kGAMEncoding;
extern NSString *const kGAMScreenColors;
extern NSString *const kGAMLanguage;
extern NSString *const kGAMJavaEnabled;
extern NSString *const kGAMFlashVersion;
extern NSString *const kGAMNonInteraction;
extern NSString *const kGAMReferrer;
extern NSString *const kGAMLocation;
extern NSString *const kGAMHostname;
extern NSString *const kGAMPage;
extern NSString *const kGAMDescription;  // synonym for kGAMScreenName
extern NSString *const kGAMScreenName;   // synonym for kGAMDescription
extern NSString *const kGAMTitle;
extern NSString *const kGAMAppName;
extern NSString *const kGAMAppVersion;

extern NSString *const kGAMEventCategory;
extern NSString *const kGAMEventAction;
extern NSString *const kGAMEventLabel;
extern NSString *const kGAMEventValue;

extern NSString *const kGAMTimingCategory;
extern NSString *const kGAMTimingVar;
extern NSString *const kGAMTimingValue;
extern NSString *const kGAMTimingLabel;

// hit types
//extern NSString *const kGAMPageView;
extern NSString *const kGAMAppView;
extern NSString *const kGAMEvent;
extern NSString *const kGAMTiming;

/*!
 This class provides several fields and methods useful as wire format parameter
 names.  The methods are used for wire format parameter names that are indexed.
 */

@interface GAMFields : NSObject

/*!
 Generates the correct parameter name for a custon dimension with an index.

 @param index the index of the custom dimension.

 @return an NSString representing the custom dimension parameter for the index.
 */
+ (NSString *)customDimensionForIndex:(NSUInteger)index;

/*!
 Generates the correct parameter name for a custom metric with an index.

 @param index the index of the custom metric.

 @return an NSString representing the custom metric parameter for the index.
 */
+ (NSString *)customMetricForIndex:(NSUInteger)index;

@end
