//
//  CoreSkitchLog.h
//  CoreSkitch
//
//  Created by Josef W. Wankerl on 2/3/14.
//  Copyright 2014 Evernote Corporation. All rights reserved.
//

#import <Foundation/Foundation.h>

@protocol CoreSkitchLogDelegate;

/**
	\brief Object to communicate diagnostic log messages from CoreSkitch.
	\details CoreSkitch generates log messages in various circumstances.  Instead of using the
		system NSLog function, the messages are instead conveyed via a delegate method so the host
		application can intercept the messages and use its own logging mechanism.
*/

@interface CoreSkitchLog : NSObject

/**
	\brief The log delegate.
	\details The deletate receives all log messages sent from CoreSkitch.
*/
@property(nonatomic, weak) id <CoreSkitchLogDelegate> delegate;

/**
	\brief Returns the shared CoreSkitchLog instance.
	\details Generally the only interesting thing to do with the sharedCoreSkitchLog instance is to set its delegate.
	\return The CoreSkitchLog object used in the CoreSkitch framework.
*/
+ (instancetype)sharedCoreSkitchLog;

@end


/**
	\brief The CoreSkitchLogDelegate protocol declares teh methods that CoreSkitch uses to communicate log messages.
*/

@protocol CoreSkitchLogDelegate <NSObject>
@required

/**
	\brief Implement this method to know when CoreSkitch sends a log message.
	\param skitchLog The CoreSkitchLog instance sending the message.
	\param message The log message.
*/
- (void)skitchLog:(CoreSkitchLog *)skitchLog message:(NSString *)message;

@end
